import azure.cosmos.cosmos_client as cosmos_client
import azure.cosmos.exceptions as exceptions

from btpy import misc


def seed_default_provider(data):
    logger = misc.setup_logging()

    client = cosmos_client.CosmosClient(
        data["cosmosdb_endpoint"], {"masterKey": data["cosmosdb_primary_key"]}
    )
    try:
        database = client.get_database_client(data["cosmosdb_database_name"])
        container = database.get_container_client("Providers")
        defaultIothub = {
            "providerType": "iothub",
            "config": {
                "transport": "http",
                "httpConfig": {"host": data["iothub_provider_service_url"]},
            },
            "displayName": "Default Iot-Hub provider",
            "id": "defaultIotHubProvider",
            "type": "Provider",
        }
        container.upsert_item(defaultIothub)
    except exceptions.CosmosHttpResponseError as e:
        logger.error("run_sample has caught an error. {0}".format(e.message))



import os

import yaml

ENV_DATA_STORAGE_NAME_ENV_VAR_KEY = "BTPY_ENV_DATA_STORAGE_NAME"
ENV_DATA_STORAGE_NAME_CONFIG_KEY = "EnvDataStorageName"


def get_env_data_storage_name():
    # Try env var
    env_value = os.getenv(ENV_DATA_STORAGE_NAME_ENV_VAR_KEY)

    if env_value:
        return env_value

    # Try user config file
    config_file_path = os.path.expanduser(os.path.normpath("~/.btpy/config.yaml"))

    if os.path.exists(config_file_path):
        with open(config_file_path, "r") as config_file:
            config_yaml = yaml.safe_load(config_file)
            config_value = config_yaml[ENV_DATA_STORAGE_NAME_CONFIG_KEY]

            if config_value:
                return config_value

    # No value found, raise error
    raise FileNotFoundError(
        f"Failed to read env data storage name from file: {config_file_path} \
        (can also be set through env var: {ENV_DATA_STORAGE_NAME_ENV_VAR_KEY})")



def parse_tf_file(file):
    """
    Parse the terraform variable file and return a list of variables.
    """
    import re

    with open(file, "r") as f:
        lines = f.readlines()
        regex = re.compile(r'variable "(?P<name>.*)" {')
        terraform_variables = [
            regex.match(line).group("name") for line in lines if regex.match(line)
        ]
        return terraform_variables



import asyncio

from azure.storage.blob.aio import BlobServiceClient

from btpy.core.azure_utility import get_azure_credential
from btpy.core.envs.env_desc import ResourceDesc
from btpy.core.resource_clients.resource_client import ResourceClient, ResourceStatus


class AzureStorageWebClient(ResourceClient):
    def __init__(self, desc: ResourceDesc):
        credential = get_azure_credential()

        self.desc = desc
        self.client = BlobServiceClient(f"https://{desc.cloud_resource.name}.blob.core.windows.net",
                                        credential=credential)

    async def close(self):
        await self.client.close()

    async def status(self):
        web_properties = await self._get_web_properties()

        return ResourceStatus.Running if web_properties.enabled else ResourceStatus.Stopped

    async def get_api_url(self):
        web_properties = await self._get_web_properties()
        return "no_url_in_web_props..."

    async def _get_web_properties(self):
        service_properties = await self.client.get_service_properties()
        return service_properties["static_website"]



import asyncio
from urllib.parse import urljoin

from azure.core.exceptions import ResourceNotFoundError
from azure.mgmt.web.aio import WebSiteManagementClient
from azure.mgmt.web.v2022_09_01.models import Site

from btpy.core.azure_utility import get_azure_credential
from btpy.core.envs.env_desc import ResourceDesc

from btpy.core.resource_clients.resource_client import ResourceClient, ResourceStatus


class AzureFunctionClient(ResourceClient):
    def __init__(self, desc: ResourceDesc):
        credential = get_azure_credential()

        self.desc = desc
        self.client = WebSiteManagementClient(credential=credential, subscription_id=desc.cloud_resource.sub_id)

    async def close(self):
        await self.client.close()

    async def status(self):
        try:
            app_client: Site = await self.client.web_apps.get(self.desc.cloud_resource.group,
                                                              self.desc.cloud_resource.name)

            return ResourceStatus.from_str(app_client.state)
        except ResourceNotFoundError:
            return ResourceStatus.Unknown

    async def get_api_url(self):
        app_client: Site = await self.client.web_apps.get(self.desc.cloud_resource.group, self.desc.cloud_resource.name)
        api_addition = f"/{self.desc.api_url_addition}" if self.desc.api_url_addition is not None else ""

        return urljoin(f"https://{app_client.default_host_name}/api", api_addition)



import os
import pathlib
import json
import logging
import tempfile
import shutil
from az.cli import az


def get_connection_string():
    """Get the connection string."""

    logger = setup_logging()
    conn_str = os.getenv("DEPLOYMENT_DATA_CONNECTION_STRING")
    if conn_str:
        return conn_str
    else:
        exit_code, result_dict, logs = az(
            'keyvault secret show --name "deployment-data-connection-string" --vault-name "btdeploymentdata" --subscription "bf222aeb-b358-4dc5-b2a3-07503773a0e3" --query "value"'
        )
        if exit_code != 0:
            logger.error(logs)
            exit(1)
        os.putenv("DEPLOYMENT_DATA_CONNECTION_STRING", result_dict)
        return result_dict


def setup_logging():
    """Setup logging."""
    log_level = os.getenv("LOG_LEVEL", "WARNING").upper()
    logger = logging.getLogger("btpy")
    logger.setLevel(log_level)
    handler = logging.StreamHandler()
    handler.setLevel(log_level)
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    return logger


def find_file_in_parent_directories(filename):
    """Find a file in the current directory or any parent directory."""
    current_directory = pathlib.Path(os.path.curdir).absolute()
    while True:
        locate_path = os.path.join(current_directory, filename)
        if os.path.exists(locate_path):
            return locate_path
        parent_directory = os.path.dirname(current_directory)
        if parent_directory == current_directory:
            raise FileNotFoundError(
                f"Could not find {filename} in any parent directory"
            )
        current_directory = parent_directory


def get_directory_of_file(filename):
    """Get the directory of a file."""
    return os.path.dirname(pathlib.Path(filename).absolute())


def write_json(filename, data):
    with open(filename, "w") as f:
        json.dump(data, f)


def create_local_temp_directory():
    """Create a local temp directory."""
    return tempfile.mkdtemp()


def remove_local_temp_directory(directory):
    """Remove a local temp directory."""
    shutil.rmtree(directory)


def sync_tfoutput_to_blob(tfname, tfoutput, data, datestamp):
    """Sync tfoutput to blob."""
    tmp_dir = create_local_temp_directory()
    cleaned_tfoutput = {k: v["value"] for k, v in tfoutput.items()}
    fname = os.path.join(tmp_dir, "output.json")
    with open(fname, "w") as f:
        f.write(json.dumps(cleaned_tfoutput, indent=4))
    latest_bloburl = f'{data["deployment_data"]}/{data["application"]}/{data["environment"]}/{data["shloc"]}{data["stamp"]}/@latest/{tfname}.json'
    datestamp_bloburl = f'{data["deployment_data"]}/{data["application"]}/{data["environment"]}/{data["shloc"]}{data["stamp"]}/{datestamp}/{tfname}.json'
    connection_string = get_connection_string()
    az(
        f"storage blob upload --overwrite --connection-string {connection_string} --blob-url {latest_bloburl} --file {fname}"
    )
    az(
        f"storage blob upload --overwrite --connection-string {connection_string} --blob-url {datestamp_bloburl} --file {fname}"
    )
    # print(
    #     f"storage blob upload --overwrite --connection-string {connection_string} --blob-url {latest_bloburl} --file {fname}"
    # )
    # print(
    #     f"storage blob upload --overwrite --connection-string {connection_string} --blob-url {datestamp_bloburl} --file {fname}"
    # )
    remove_local_temp_directory(tmp_dir)


def sync_env_to_blob(tfname, data, datestamp):
    """Sync tfoutput to blob."""
    tmp_dir = create_local_temp_directory()
    fname = os.path.join(tmp_dir, "output.json")
    with open(fname, "w") as f:
        f.write(json.dumps(data, indent=4))
    latest_bloburl = f'{data["deployment_data"]}/{data["application"]}/{data["environment"]}/{data["shloc"]}{data["stamp"]}/@latest/{tfname}-environment-data.json'
    datestamp_bloburl = f'{data["deployment_data"]}/{data["application"]}/{data["environment"]}/{data["shloc"]}{data["stamp"]}/{datestamp}/{tfname}-environment-data.json'
    connection_string = get_connection_string()
    az(
        f"storage blob upload --overwrite --connection-string {connection_string} --blob-url {latest_bloburl} --file {fname}"
    )
    az(
        f"storage blob upload --overwrite --connection-string {connection_string} --blob-url {datestamp_bloburl} --file {fname}"
    )
    # print(
    #     f"storage blob upload --overwrite --connection-string {connection_string} --blob-url {latest_bloburl} --file {fname}"
    # )
    # print(
    #     f"storage blob upload --overwrite --connection-string {connection_string} --blob-url {datestamp_bloburl} --file {fname}"
    # )
    remove_local_temp_directory(tmp_dir)



from btpy.core.resource_clients.resource_client import ResourceStatus


def print_status(status: ResourceStatus):
    if status == ResourceStatus.Running:
        color = "[green]"
    elif status == ResourceStatus.Stopped:
        color = "[red]"
    elif status == ResourceStatus.Restarting or status == ResourceStatus.Unknown or status == ResourceStatus.Mixed:
        color = "[yellow]"
    else:
        raise ValueError(f"Unknown status: {status}")

    return f"{color}{status.name}"



import asyncio

from azure.mgmt.compute.aio import ComputeManagementClient
from azure.mgmt.compute.v2023_09_01.models import VirtualMachine

from btpy.core.azure_utility import get_azure_credential
from btpy.core.envs.env_desc import ResourceDesc
from btpy.core.resource_clients.resource_client import ResourceClient, ResourceStatus


class AzureVmClient(ResourceClient):
    def __init__(self, desc: ResourceDesc):
        credential = get_azure_credential()

        self.desc = desc
        self.management_client = ComputeManagementClient(credential=credential,
                                                         subscription_id=desc.cloud_resource.sub_id)

    async def close(self):
        await self.management_client.close()

    async def status(self):
        vm_client: VirtualMachine = await self.management_client.virtual_machines.get(self.desc.cloud_resource.group,
                                                                                      self.desc.cloud_resource.name)

        # return vm_client.instance_view.statuses  # TODO: Check if this even works
        return ResourceStatus.Unknown



import time
from az.cli import az
from rich.progress import Progress, TimeElapsedColumn

from btpy import misc

logger = misc.setup_logging()


def get_providers():
    exit_code, result_dict, logs = az("provider list")
    if exit_code != 0:
        logger.error(logs)
        exit(1)
    result_dict = [x for x in result_dict if "namespace" in x]
    return result_dict


def check_if_registered(
    provider_name,
):
    providers = get_providers()
    provider = [x for x in providers if x["namespace"] == provider_name]
    print(provider[0]["registrationState"])
    return "Registered" == provider[0]["registrationState"]


def register_provider(provider_name):
    """ """
    timeout = 300  # 5 minutes
    interval = 5  # 5 seconds
    elapsed_time = 0

    exit_code, result_dict, logs = az(f"provider register --namespace {provider_name}")
    if exit_code != 0:
        logger.error(logs)
        exit(1)
    elif logs:
        logger.info(logs)
    with Progress(
        *Progress.get_default_columns(),
        TimeElapsedColumn(),
    ) as progress:
        task = progress.add_task(
            f"Registering provider {provider_name}",
            filename=provider_name,
            start=True,
            total=None,
        )
        while elapsed_time < timeout:
            exit_code, result_dict, logs = az(f"provider show -n {provider_name}")
            if exit_code != 0:
                logger.error(logs)
                exit(1)
            elif logs:
                logger.info(logs)

            if result_dict["registrationState"] == "Registered":
                logger.info(f"{provider_name} registration completed")
                return result_dict

            time.sleep(interval)
            progress.update(task)
            elapsed_time += interval

    # return result_dict
    logger.error(f"{provider_name} registration timed out")
    exit(1)



from enum import Enum


class ResourceStatus(Enum):
    Running = 0,
    Stopped = 1,
    Restarting = 2,
    Mixed = 3,
    Unknown = 100

    @staticmethod
    def from_str(status: str):
        status_lower = status.lower()

        if status_lower == "running":
            return ResourceStatus.Running
        elif status_lower == "stopped":
            return ResourceStatus.Stopped
        elif status_lower == "restarting":
            return ResourceStatus.Restarting
        elif status_lower == "mixed":
            return ResourceStatus.Mixed
        else:
            return ResourceStatus.Unknown


class ResourceClient:
    async def close(self):
        raise NotImplementedError()

    async def status(self) -> ResourceStatus:
        raise NotImplementedError()

    async def get_log_level(self) -> str:
        raise NotImplementedError()

    async def set_log_level(self, log_level: str):
        raise NotImplementedError()

    async def get_quick_info(self):
        raise NotImplementedError()

    async def get_api_url(self) -> str:
        raise NotImplementedError()



import asyncio

import typer
from colorama import Fore
from rich.console import Console
from rich.live import Live
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.spinner import Spinner
from rich.table import Table

from btpy.core.envs import env_desc_loader
from btpy.core.envs.env_client import EnvClient
from btpy.core.resource_clients.resource_client import ResourceStatus
from btpy.print_utility import print_status

app = typer.Typer()


@app.command("list")
def list_envs_cmd():
    asyncio.run(_list_envs())


async def _list_envs():
    env_list = await env_desc_loader.list_envs()

    for env in env_list:
        print(env)


@app.command("status")
def env_status_cmd(env_name: str):
    asyncio.run(_get_env_status(env_name))


async def _get_env_status(env_name: str):
    env_desc = await env_desc_loader.load_env_desc(env_name)

    if env_desc is None:
        print("Env not found")
        return

    env_client = EnvClient(env_desc)
    resource_clients = env_client.get_resource_clients()

    with Progress(SpinnerColumn(), TextColumn("{task.description}"), TextColumn("{task.fields[result]}")) as table:
        loop = asyncio.get_event_loop()
        tasks = [(resource_client, loop.create_task(_get_formatted_status(resource_client.client)),
                  table.add_task(f"[blue]{resource_client.name}[/blue]", result="")) for resource_client in
                 resource_clients]

        while not table.finished:
            for (resource_client, task, table_task) in tasks:
                if task.done():
                    table.update(table_task, total=100, completed=100, result=task.result())

            await asyncio.sleep(0.1)

    [await resource_client.client.close() for resource_client in resource_clients]


async def _get_formatted_status(resource_client):
    status = await resource_client.status()
    return print_status(status)



from azure.identity import AzureCliCredential, DefaultAzureCredential


def get_azure_credential():
    return AzureCliCredential()
    # return DefaultAzureCredential()


def to_azure_appsetting_format(setting_value: str) -> str:
    return setting_value.replace(":", "__")


def from_azure_appsetting_format(setting_value: str) -> str:
    return setting_value.replace("__", ":")
